# Perform checks that can only be performed at the end

# Input: the global information environment
#
# Output: NULL
#
# Details: returns errors or warnings based on java/shiny requirements
#
finalChecks = function(globalInfo) {
  # Check internal links
  links = getGlobal("internalLinks", globalInfo)
  if (!is.null(links)) {
    locs = getGlobal("internalLocs", globalInfo)
    if (is.null(locs)) {
      stop("@", links[1,"lineNum"], " \\shinyInternalLink, but app has no \\shinyInternalLoc's")
    }
    for (i in 1:nrow(links)) {
      if (is.na(match(links[i, "name"], locs))) {
        stop("@", links[i, "lineNum"], " \\shinyInternalLink to '", links[i, "name"], 
             "' has no matching \\shinyInternalLoc")
      }
    }
  } # end checking internal links

  # Check the conditional panel conditions are valid
  inputIds = getGlobal("inputIds", globalInfo)
  cps = getGlobal("conditionalPanels", globalInfo)
  if (!is.null(cps)) {
    for (cp in 1:nrow(cps)) {
      cId = cps[cp, "inputId"]
      pos = match(cId, inputIds$inputId)
      if (is.na(pos)) {
        stop("@", cps[cp, "lineNum"], " \\shinyConditionalPanel references missing ",
             "inputId='", cId, "'")
      }
      cType = inputIds[pos, "type"]
      ## more checks here !!!!!!!!!! ??????????????
      if (cType == "shinyDropdown") {
        if (!exists("dropdownChoices")) dropdownChoices = getGlobal("dropdownChoices", globalInfo)
        cChoicesList = dropdownChoices[[cId]]
        assure(!is.null(cChoicesList), 77926501)
        cChoices = unlist(gsub("[ ]*$", "", gsub("^[ ]*", "", 
                        strsplit(cChoicesList[["choices"]], ","))))
        if (is.na(match(cps[cp, "condition"], cChoices))) {
          stop("@", cps[cp, "lineNum"], " \\shinyConditionalPanel reference value '",
               cps[cp, "condition"], "', but that is not one of the dropdown choices on line ",
               inputIds[pos, "lineNum"])
        }
      }
    }
  }
  
  # Check that files are present
  filenames = getGlobal("filenames", globalInfo)
  if (!is.null(filenames)) {
    for (i in 1:nrow(filenames)) {
      if (!file.exists(file.path("www", filenames[i, "filename"]))) {
        warning("expecting ", filenames[i, "type"], " ", filenames[i, "filename"], 
                " in 'www' folder")
      }
    }
  }

  invisible(NULL)
}
